﻿//Order Book Reconstruction written by Martin Sewell martin@martinsewell.com

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace OrderBook
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            loadButton.Enabled = true;
            constructButton.Enabled = false;
            saveButton.Enabled = false;
        }

        string[][] sdata;
        int noLines;
        const int orderBookDepth = 2 * 5;
        Order[,] orderBook;

        private void loadButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 2;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    using (StreamReader sr = new StreamReader(openFileDialog1.FileName))
                    {
                        fileLabel.Text = openFileDialog1.FileName;
                        fileLabel2.Text = "";
                        saveButton.Enabled = false;
                        constructButton.Enabled = true;

                        noLines = 0;
                        int ln = 0;
                        string lin = "";

                        while ((lin = sr.ReadLine()) != null)
                            if (lin.Length > 0)
                        {
                            noLines++;
                        }
                        sr.BaseStream.Seek(0, SeekOrigin.Begin);
                        string line = "";

                        sdata = new string[noLines][];

                        while ((line = sr.ReadLine()) != null)
                            if (line.Length > 0)
                        {
                            sdata[ln] = line.Split(' ');
                            ln++;
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: Could not read file from disk. Original error: " + ex.Message);
                }
            }
        }



        public class Order
        {
            public int Price;
            public int Volume;

            public Order(int p, int v)
            {
                Price = p;
                Volume = v;
            }
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            FileStream output;

            SaveFileDialog fileChooser = new SaveFileDialog();

            fileChooser.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            //fileChooser.FilterIndex = 2;

            DialogResult result = fileChooser.ShowDialog();
            string fileName;
            fileChooser.CheckFileExists = false;
            if (result == DialogResult.Cancel)
                return;
            fileName = fileChooser.FileName;
            if (fileName == "" || fileName == null)
                MessageBox.Show("Invalid File Name", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                try
                {
                    output = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.Write);
                    StreamWriter fileWriter = new StreamWriter(output);
                    for (int i = 0; i < noLines; i++)
                    {
                        for (int j = 0; j < orderBookDepth; j++)
                        {
                            if (orderBook[i, j] != null)
                            {


                                if ((j == 4) || (j == 5)) //delete this!
                                {

                                    if (j <= 4)
                                    {
                                        fileWriter.Write("A");
                                        fileWriter.Write(5 - j);
                                    }
                                    else
                                    {
                                        fileWriter.Write("B");
                                        fileWriter.Write(j - 4);
                                    }
                                    fileWriter.Write(": ");

                                    fileWriter.Write((orderBook[i, j].Price));
                                    fileWriter.Write(" ");
                                    fileWriter.Write((orderBook[i, j].Volume));
                                    if (j < orderBookDepth - 1)
                                        fileWriter.Write(" ");
                                }
                            }
                        }
                        fileWriter.Write("\n");
                    }
                    fileWriter.Close();
                }
                catch (IOException)
                {
                    MessageBox.Show("Error opening file", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void constructButton_Click(object sender, EventArgs e)
        {
            fileLabel2.Text = "Constructing...";
            orderBook = new Order[noLines, orderBookDepth];  //noLines is far larger than necessary

            int row; //order book row
            float p;
            for (int i = 0; i < noLines; i++)
            {
                p = (float)(100*i / noLines);
                fileLabel2.Text = String.Format("{0:0%}", p/100);
                this.Refresh();
                {
                    row = 0;
                    switch ( sdata[i][0] )
                    {
                        case "A":
                            while (orderBook[row, 5 - Convert.ToInt32(sdata[i][1])] != null) row++;
                            orderBook[row, 5 - Convert.ToInt32(sdata[i][1])] = new Order(Convert.ToInt32(sdata[i][2]), Convert.ToInt32(sdata[i][3])); 
                        break;
                        case "B":
                            while (orderBook[row, Convert.ToInt32(sdata[i][1]) + 4] != null) row++;
                            orderBook[row, Convert.ToInt32(sdata[i][1]) + 4] = new Order(Convert.ToInt32(sdata[i][2]), Convert.ToInt32(sdata[i][3]));        
                        break;
                        default:
                        break;
                    }
                }
            }
            fileLabel2.Text = "Finished";
            saveButton.Enabled = true;
        }
    }
}
